/*
 *  @(#)LudgerSynth.java	1.15	99/12/03
 *
 *  This program is based on one of Suns Java Sound Demos which contains the
 *  following copyright notice:
 *
 *  Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 *  Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 *  modify and redistribute this software in source and binary code form,
 *  provided that i) this copyright notice and license appear on all copies of
 *  the software; and ii) Licensee does not utilize the software in a manner
 *  which is disparaging to Sun.
 *
 *  This software is provided "AS IS," without a warranty of any kind. ALL
 *  EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 *  IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 *  NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 *  LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 *  OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 *  LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 *  INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 *  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 *  OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGES.
 *
 *  This software is not designed or intended for use in on-line control of
 *  aircraft, air traffic, aircraft navigation or aircraft communications; or in
 *  the design, construction, operation or maintenance of any nuclear
 *  facility. Licensee represents and warrants that it will not use or
 *  redistribute the Software for such purposes.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.sound.midi.*;
import java.util.Vector;
import java.io.File;
import java.io.IOException;

/**
 *  Illustrates general MIDI melody instruments and MIDI controllers.
 *
 *@author                 Brian Lichtenwalter
 *@created                16. Juli 2003
 *@version
 *@(#)LudgerSynth.java    1.15 99/12/03
 */
public class LudgerSynth extends JPanel implements ControlContext, ActionListener {

    final int PROGRAM = 192;
    final int NOTEON = 144;
    final int NOTEOFF = 128;
    final int ON = 0, OFF = 1;
    int velocity = 100;
    Sequencer sequencer;
    Sequence sequence;
    Synthesizer synthesizer;
    Instrument instruments[];
    JPanel p1, p2, p3, pp;
    JLabel name;
    JTextField Sound;
    JButton playButton, clearButton, calculateButton;
    Vector keys = new Vector();
    Vector whiteKeys = new Vector();
    Piano piano;
    Track track;
    TempoDial tempoDial = new TempoDial();
    JTextArea results;
    JScrollPane holder;
    JMenu match;
    JMenuItem first, second, third, fourth, fifth;
    JMenuBar mb;
    Image logo;
   
    String Bass = "off", Bassnote;
    int b;
    double factor, sumf, sump, sonance, vp;
    String[] pitch =  {"c", "c#", "d", "d#", "e", "f", "f#", "g", "g#",
                      "a", "a#", "b", "c", "c#", "d", "d#", "e", "f", "f#", "g", "g#", "a", "a#", "b", "c"};
    int[] keyNumber ={1, 3, 6, 8, 10, 13, 15, 18, 20, 22, 0, 2, 4, 5, 7, 9, 11, 12, 14, 16, 17, 19, 21, 23, 24};
    int[] inverseKeyNumber = {10, 0, 11, 1, 12, 13, 2, 14, 3, 15, 4, 16, 17, 5, 18, 6, 19, 20, 7, 21, 8, 22, 9, 23, 24};
    String[] newPitch = new String[25];
    String[] pitchPrint = new String[25];
    double[][] virtualPitch = new double [25][6];
    double[] virtuality = new double [25];
    double[] newVirtuality = new double [25];
    double[] virtualityPrint = new double [25];
    double[] virtualityFinal = new double[25];


    /**
     *  Constructor for the LudgerSynth object
     */
    public LudgerSynth() 
    
    {
        
     
        setLayout(new BorderLayout());

        JPanel p = new JPanel();
        p.setLayout(new BorderLayout(1,1));
        EmptyBorder eb = new EmptyBorder(29, 5, 5, 5);
        BevelBorder bb = new BevelBorder(BevelBorder.LOWERED);
        CompoundBorder cb = new CompoundBorder(eb, bb);
        p.setBorder(new CompoundBorder(cb, eb));
        p.setBackground(Color.blue);
        
        mb= new JMenuBar();

	Sound = new JTextField(4);
       

        p1 = new JPanel();
        p1.setLayout(new FlowLayout());
        p1.setBackground(Color.blue);
      

        
        p1.add("East", playButton = new JButton("play"));
        playButton.addActionListener(this);
        p1.add("East", calculateButton = new JButton("calculate"));
        calculateButton.addActionListener(this);
        p1.add("East", clearButton = new JButton("clear"));
        p1.add("Center", name = new JLabel("           copyright Hofmann-Engl 2012"));
        clearButton.addActionListener(this);
        
       
        
        
        
        JPanel pp = new JPanel(new BorderLayout());
        pp.setBorder(new EmptyBorder(10, 20, 10, 5));
        pp.add(piano = new Piano());
        

        results = new JTextArea("                Results Window", 18, 50);
        holder = new JScrollPane(results);
        Font f1 = new Font("Courier", Font.BOLD, 11);
        results.setFont(f1);
        results.setEditable(false);
        p2 = new JPanel();
        p2.setLayout(new FlowLayout());
        p2.add(holder);
        p2.setBackground(Color.blue);
        
        p3 = new JPanel();
        p3.setLayout(new FlowLayout());
        p3.setBackground(Color.blue);
        
        match = new JMenu("Matching Bass Notes");
        p3.add("East",mb);
        p3.add("West",Sound);

        (first = new JMenuItem("Best Match")).addActionListener(this);
        match.add(first);
        (second = new JMenuItem("Second best Match")).addActionListener(this);;
        match.add(second);
        (third = new JMenuItem("Third best Match")).addActionListener(this);;
        match.add(third);
        (fourth = new JMenuItem("Fourth best Match")).addActionListener(this);;
        match.add(fourth);
        (fifth = new JMenuItem("Fifth best Match")).addActionListener(this);;
        match.add(fifth);
        
        mb.add(match);
        

        p.add("West", p1);
        p.add("South",pp);
        p.add("North",p2);
        p.add("East", p3);

        add(p);
    }


    /**
     *  Opening and initializing the Midi system
     */
    public void open() {
        try {
            if (synthesizer == null) {
                if ((synthesizer = MidiSystem.getSynthesizer()) == null) {
                    System.out.println("getSynthesizer() failed!");
                    return;
                }
            }
            synthesizer.open();
            sequencer = MidiSystem.getSequencer();
            sequencer.open();
        } catch (Exception ex) {
            ex.printStackTrace();
            return;
        }

        Soundbank sb = synthesizer.getDefaultSoundbank();
        if (sb != null) {
            instruments = synthesizer.getDefaultSoundbank().getInstruments();
            for (int i = 0; i < instruments.length; i++) {
               if (instruments[i].getName().contains("Piano")) {
                  synthesizer.loadInstrument(instruments[i]);
                  break;
               } else if (i == instruments.length - 1) {
                  System.out.println("Soundbank does not contain instrument Piano");
                  System.exit(0);
               }
            }
        }
    }


    /**
     *  Closing the Midi system
     */
    public void close() {
        if (synthesizer != null) {
            synthesizer.close();
        }
        if (sequencer != null) {
            sequencer.close();
        }
        sequencer = null;
        synthesizer = null;
    }


    /**
     *  Creating a Midi Event and appending it to the track
     *
     *@param  type  type of Midi command
     *@param  chan  number of channel
     *@param  num   first Midi data byte
     *@param  tick  timing information
     */
    private void createEvent(int type, int chan, int num, long tick) {
        ShortMessage message = new ShortMessage();
        try {
            message.setMessage(type, chan, num, velocity);
            MidiEvent event = new MidiEvent(message, tick);
            track.add(event);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    /**
     *  Building and starting the sequence for playing the marked
     *  piano keys simultaniously
     */
    private void buildTrackThenStartSequencer(int kNum) {
        Sequence sequence = null;
        try {
            sequence = new Sequence(Sequence.PPQ, 4);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        track = sequence.createTrack();
        createEvent(PROGRAM, 0, 1, 0);
                createEvent(NOTEON, 0, kNum, 0);
                createEvent(NOTEOFF, 0, kNum, 20);
       // 0 is starting point 20 is end
        createEvent(PROGRAM, 0, 1, 20);

        // set and start the sequencer.
        try {
            sequencer.setSequence(sequence);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        sequencer.start();
        sequencer.setTempoInBPM(tempoDial.getTempo());
    }

    /**
     *  Building and starting the sequence for playing the marked
     *  piano keys simultaniously
     */
    private void buildTrackThenStartSequencer() {
        Sequence sequence = null;
        try {
            sequence = new Sequence(Sequence.PPQ, 4);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        track = sequence.createTrack();
        createEvent(PROGRAM, 0, 1, 0);
        
       if(Bass == "on")
       {
           if(Bassnote == "c")
           {
              b = 36;
           }           
           
           if(Bassnote == "c#")
           {
               b = 37;
           }
           
           if(Bassnote == "d")
           {
               b = 38;
           }
           
           if(Bassnote == "d#")
           {
               b = 39;
           }
           
            if(Bassnote == "e")
           {
              b = 40;
           }           
           
           if(Bassnote == "f")
           {
               b = 41;
           }
           
           if(Bassnote == "f#")
           {
               b = 42;
           }
           
           if(Bassnote == "g")
           {
               b = 43;
           }
           
           if(Bassnote == "g#")
           {
              b = 44;
           }           
           
           if(Bassnote == "a")
           {
               b = 45;
           }
           
           if(Bassnote == "a#")
           {
               b = 46;
           }
           
           if(Bassnote == "b")
           {
               b = 47;
           }
           
            createEvent(NOTEON, 0, b, 5);
            createEvent(NOTEOFF, 0, b, 20);
            
       }
        
       
           
        for (int i = 0; i < keys.size(); i++) {
            if (((Key) (keys.get(i))).isNoteOn()) {
                createEvent(NOTEON, 0, ((Key) (keys.get(i))).kNum, 0);
                createEvent(NOTEOFF, 0, ((Key) (keys.get(i))).kNum, 20);
            }
            
            
        }
        // so we always have a track from 0 to 10.
        createEvent(PROGRAM, 0, 1, 20);

        // set and start the sequencer.
        try {
            sequencer.setSequence(sequence);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        Bass = "off";
        sequencer.start();
        sequencer.setTempoInBPM(tempoDial.getTempo());
    }

    /**
     *  Action listener
     *
     *@param  press  Action event
     */
    public void actionPerformed(ActionEvent press) {
        if (press.getSource() == playButton) {
            buildTrackThenStartSequencer();
        } 
 
        else if (press.getSource() == first)
            
        {
            Bass = "on";
            Bassnote = pitchPrint[0];
	    Sound.setText(pitchPrint[0]+" (1st)");
            
            
            buildTrackThenStartSequencer();
        } 
        
         else if (press.getSource() == second)
            
        {
            Bass = "on";
            Bassnote = pitchPrint[1];
            Sound.setText(pitchPrint[1]+" (2nd)");
            
            
            buildTrackThenStartSequencer();
        } 
        
         else if (press.getSource() == third)
            
        {
            Bass = "on";
            Bassnote = pitchPrint[2];
            Sound.setText(pitchPrint[2]+" (3rd)");
            
            
            buildTrackThenStartSequencer();
        } 
        
         else if (press.getSource() == fourth)
            
        {
            Bass = "on";
            Bassnote = pitchPrint[3];
            Sound.setText(pitchPrint[3]+" (4th)");
            
            
            buildTrackThenStartSequencer();
        } 
        
         else if (press.getSource() == fifth)
            
        {
            Bass = "on";
            Bassnote = pitchPrint[4];
            Sound.setText(pitchPrint[4]+" (5th)");
            
            
            buildTrackThenStartSequencer();
        } 
        
        else if (press.getSource() == clearButton) {
            for (int i = 0; i < keys.size(); i++) {
                ((Key) (keys.get(i))).off();
            }
            repaint();
        } else if (press.getSource() == calculateButton) {
            int j = 0;
            int n = 0;
            sumf = 0;
            sump = 0;
            double m = 0;

            for (int i = 0; i < 25; i++) {
                
                //i to how many notes there are
                
                virtuality[i] = 0;
                newVirtuality[i] = 0;
                virtualityPrint[i] = 0;
                newPitch[i] = "";
                pitchPrint[i] = "";
                for (int k = 0; k < 6; k++)
                   virtualPitch[i][k] = 0;
            }

            Font f2 = new Font("Utopia", Font.PLAIN, 12);
            results.setFont(f2);
            results.append("\n");
            results.append("\n");
            results.append("Data for the chord: ");
            for (int i = 0; i < 25; i++) {
                int k = inverseKeyNumber[i];
                if (((Key)(keys.get(k))).isNoteOn()) {
                    results.append(pitch[keyNumber[k]] + " ");
                    j++;
                }
            }
            results.append("\n");
            int p = 0;
            for (int i = 0; i < 25; i++) {
                
                //i to how many notes there are
                
                
                int k = inverseKeyNumber[i];
                if (((Key)(keys.get(k))).isNoteOn()) {
                    virtualPitch[p][0] = Math.IEEEremainder(i, 12);
                    if (virtualPitch[p][0] < 0) {
                        virtualPitch[p][0] = 12 + virtualPitch[p][0];
                    }
                    System.out.println("virtualPitch[" + p + ",0] = " + virtualPitch[p][0]);
                    virtualPitch[p][1] = Math.IEEEremainder(i + 5, 12);
                    if (virtualPitch[p][1] < 0) {
                        virtualPitch[p][1] = 12 + virtualPitch[p][1];
                    }
                    System.out.println("virtualPitch[" + p + ",1] = " + virtualPitch[p][1]);
                    virtualPitch[p][2] = Math.IEEEremainder(i + 8, 12);
                    if (virtualPitch[p][2] < 0) {
                        virtualPitch[p][2] = 12 + virtualPitch[p][2];
                    }
                    System.out.println("virtualPitch[" + p + ",2] = " + virtualPitch[p][2]);
                    virtualPitch[p][3] = Math.IEEEremainder(i + 2, 12);
                    if (virtualPitch[p][3] < 0) {
                        virtualPitch[p][3] = 12 + virtualPitch[p][3];
                    }
                    System.out.println("virtualPitch[" + p + ",3] = " + virtualPitch[p][3]);
                    virtualPitch[p][4] = Math.IEEEremainder(i + 10, 12);
                    if (virtualPitch[p][4] < 0) {
                        virtualPitch[p][4] = 12 + virtualPitch[p][4];
                    }
                    System.out.println("virtualPitch[" + p + ",4] = " + virtualPitch[p][4]);
                    virtualPitch[p][5] = Math.IEEEremainder(i + 1, 12);
                    if (virtualPitch[p][5] < 0) {
                        virtualPitch[p][5] = 12 + virtualPitch[p][5];
                    }
                    System.out.println("virtualPitch[" + p + ",5] = " + virtualPitch[p][5]);
                    p++;
                }
            }
            for (int i = 0; i < 12; i++) {
                m = 0;
                for (int k = 0; k < j; k++) {
                    m++;
                    for (int l = 0; l < 6; l++) {
                        if (i == virtualPitch[k][l]) {
                            virtuality[i] = virtuality[i] + ((36 -
                                    Math.pow(l, 2)) * Math.sqrt(1.0 / m));
                            System.out.println("* k = " + k + ", l = " + l +
                                 ", virtuality[" + i + "] = " + virtuality[i]);
                         }
                    }
                }
                virtuality[i] = virtuality[i] / (j * 6);
                System.out.println("virtuality[" + i + "] = " + virtuality[i]);
            }
            for (int i = 0; i < 12; i++) {
                if (virtuality[i] != 0) {
                    newVirtuality[n] = virtuality[i];
                    newPitch[n] = pitch[i];
                    System.out.println("newPitch[" + n + "] = " + newPitch[n]);
                    n++;
                }
            }
            for (int i = 0; i < n; i++) {
                int o = 0;
                for (int k = 0; k < n; k++) {
                    if (newVirtuality[i] < newVirtuality[k]) {
                        o++;
                    }
                }
                virtualityFinal[o] = newVirtuality[i];
                virtualityPrint[o] = Math.round(100 * newVirtuality[i]);
                pitchPrint[o] = newPitch[i];
            }
            for (int i = 0; i < n; i++) {
                results.append("Virtuality of: " + pitchPrint[i] + " = " +
                               Double.toString(virtualityPrint[i]/100) + " Hh" + "\n");
            }
            m = 0;
            for (int i = 0; i < j; i++) {
                m++;
                sumf = sumf + Math.sqrt(1 / m);
            }
            System.out.println("sumf = " + sumf);
            factor = j / sumf;
            for (int i = 0; i < n; i++) {
                sump = sump + virtualityFinal[i];
            }
            vp = virtualityFinal[0] / sump;
            sonance = factor * (virtualityFinal[0] / (6 + Math.pow(n, 2)
               / 6 * Math.sqrt(Math.abs(1 - Math.pow(vp / 0.2236024844720497, 2)))));
            sonance = Math.round(1000*sonance);
            results.append("Sonance Factor is: " + Double.toString(sonance/1000) + " Sh" + "\n" + "\n");
for (int i = 0; i <= n; i++) {
                newVirtuality[i] = 0;
            }
        }
    }



    /**
     *  Class for keys. Allows to change the state of the key
     *  and to ask for its state.
     *
     *@author     fhofmann
     *@created    16. Juli 2003
     */
    class Key extends Rectangle {
        int noteState = OFF;
        int kNum;


        /**
         *  Constructor for the Key object
         *
         *@param  x       x coordinate for painting
         *@param  y       y coordinate for painting
         *@width  width   width of key
         *@param  height  height of key
         *@param  num     note number
         */
        public Key(int x, int y, int width, int height, int num) {
            super(x, y, width, height);
            kNum = num;
        }


        /**
         *  Gets the noteOn attribute of the Key object
         *
         *@return    The noteOn value
         */
        public boolean isNoteOn() {
            return noteState == ON;
        }


        /**
         *  Sets state to ON
         */
        public void on() {
            setNoteState(ON);
        }


        /**
         *  Sets state to OFF
         */
        public void off() {
            setNoteState(OFF);
        }


        /**
         *  Sets the noteState attribute of the Key object
         *
         *@param  state  The new noteState value
         */
        public void setNoteState(int state) {
            noteState = state;
        }
    }
    // End class Key



    /**
     *  Piano renders black & white keys and plays the notes for a MIDI channel.
     *@ author ludger hofmannn-engl, 2002,2004,2008 (algorithm, concept, octave extension, bass notes (sounds + names))
     *@ author     fhofmann (changing gui to object) 2003
     *
     */
    class Piano extends JPanel implements MouseListener {

        Vector blackKeys = new Vector();
        Key prevKey;
        int keyNum;
        final int kw = 40, kh = 150;


        /**
         *  Constructor for the Piano object
         */
        public Piano() {
            setLayout(new BorderLayout());
            setPreferredSize(new Dimension(42 * kw, kh + 1));
            int transpose = 60;
            int whiteIDs[] = {0, 2, 4, 5, 7, 9, 11, 12, 14, 16, 17, 19, 21, 23, 24};
            
            //add ids accoring to key number
            
            int x = 0;
            for (int j = 0; j < 14; j++, x += kw) 
            
                //j to amount of white keys
                
            {
                int keyNum = whiteIDs[j] + transpose;
                whiteKeys.add(new Key(x, 0, kw, kh, keyNum));
            }
            keyNum = 24 + whiteIDs[0] + transpose;
            
            //24 tells how many notes all together - all in all 24+1
            
            whiteKeys.add(new Key(x, 0, kw, kh, keyNum));
            x = 0;
            keyNum = transpose;
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 1));
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 3));
            x += kw;
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 6));
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 8));
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 10));
            x += kw;
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 13));
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 15));
            x += kw;
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 18));
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 20));
            blackKeys.add(new Key((x += kw) - kw / 4, 0, kw / 2, (int) (kh * 0.6), keyNum + 22));
          
            keys.addAll(blackKeys);
            keys.addAll(whiteKeys);

            addMouseMotionListener(
                new MouseMotionAdapter() {
                    /**
                     *  Description of the Method
                     *
                     *@param  e  Description of the Parameter
                     */
                    public void mouseMoved(MouseEvent e) { }
                });
            addMouseListener(this);
        }


        /**
         *  When key is pressed toggle its state and play the marked keys
         *
         *@param  e  Mouse event
         */
        public void mousePressed(MouseEvent e) {
            prevKey = getKey(e.getPoint());
            if (prevKey != null) {
                prevKey.setNoteState(prevKey.isNoteOn() ? OFF : ON);
                repaint();
                buildTrackThenStartSequencer(prevKey.kNum);
            }
        }


        /**
         *  Do nothing on MouseReleased
         *
         *@param  e  Description of the Parameter
         */
        public void mouseReleased(MouseEvent e) { }


        /**
         *  Do nothing on MouseExited
         *
         *@param  e  Description of the Parameter
         */
        public void mouseExited(MouseEvent e) { }


        /**
         *  Do nothing on MouseClicked
         *
         *@param  e  Description of the Parameter
         */
        public void mouseClicked(MouseEvent e) { }


        /**o nothing on MouseEntered
         *
         *@param  e  Description of the Parameter
         */
        public void mouseEntered(MouseEvent e) { }


        /**
         *  Gets the key that's pressed
         *
         *@param  point  pressed point
         *@return        the key pressed
         */
        public Key getKey(Point point) {
            for (int i = 0; i < blackKeys.size()+1; i++) {
                if (((Key) keys.get(i)).contains(point)) {
                    return (Key) keys.get(i);
                }
                
                //+1 if the scale ends on white key
            }
                if (((Key) keys.get(blackKeys.size()-1)).contains(point)) {
                    return null;
                }
            
            // -1 if the bottom ends on white key
            
            for (int i = blackKeys.size()+1; i < keys.size(); i++) {
                if (((Key) keys.get(i)).contains(point)) {
                    return (Key) keys.get(i);
                }
            }
            return null;
        }

        /**
         *  Paint piano and states of the keys
         *
         *@param  g  graphics object
         */
        public void paint(Graphics g) {
            Graphics2D g2 = (Graphics2D) g;
            Dimension d = getSize();

            g2.setBackground(getBackground());
            g2.clearRect(0, 0, d.width, d.height);

            g2.setColor(Color.white);
            g2.fillRect(0, 0, 15 * kw, kh);
            
            //number 10 tells how many white keys are there

            for (int i = 0; i < whiteKeys.size(); i++) {
                Key key = (Key) whiteKeys.get(i);
                g2.setColor(Color.black);
                g2.draw(key);
                if (key.isNoteOn()) {
                    g2.setColor(Color.red);
                    g2.fillOval((int) (key.getX() + key.getWidth() / 2 - 10),
                            (int) (key.getY() + key.getHeight() - 50),
                            20, 20);
                }
            }
            for (int i = 0; i < blackKeys.size(); i++) {
                Key key = (Key) blackKeys.get(i);
                g2.setColor(Color.black);
                g2.fill(key);
                if (key.isNoteOn()) {
                    g2.setColor(Color.red);
                    g2.fillOval((int) (key.getX() + key.getWidth() / 2 - 10),
                            (int) (key.getY() + key.getHeight() - 50),
                            20, 20);
                }
            }
        }
    }
    // End class Piano


    /**
     * Main method
     *
     */
    public static void main(String args[]) {
        final LudgerSynth LudgerSynth = new LudgerSynth();
        LudgerSynth.open();
        JFrame f = new JFrame("Ludgers Synthesizer");
        f.addWindowListener(
            new WindowAdapter() {
                /**
                 *  Description of the Method
                 *
                 *@param  e  Description of the Parameter
                 */
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            });
        f.getContentPane().add("Center", LudgerSynth);
        f.pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 760;
        int h = 590;
        f.setLocation(screenSize.width / 2 - w / 2, screenSize.height / 2 - h / 2);
        f.setSize(w, h);
        f.setVisible(true);
    }
}


